@@ -1,13 +1,12 @@
 //! hatanaka.rs   
 //! structures and macros suites for
 //! RINEX OBS files compression and decompression.   
+use crate::sv;
 use crate::header;
-use crate::record;
 use crate::is_comment;
 use crate::types::Type;
 use thiserror::Error;
 use std::str::FromStr;
-use crate::record::Sv;
 use std::collections::HashMap;
 
 #[derive(Error, Debug)]
@@ -203,7 +202,7 @@ pub enum Error {
     #[error("CRINEX3 standard mismatch")]
     FaultyCrinex3Format,
     #[error("failed to identify sat. vehicule")]
-    ParseSvError(#[from] record::ParseSvError),
+    SvError(#[from] sv::Error),
     #[error("failed to parse integer number")]
     ParseIntError(#[from] std::num::ParseIntError),
     #[error("data recovery failed")]
@@ -227,7 +226,7 @@ pub struct Decompressor {
     /// Clock offset decompressor
     clk_krn : Kernel,
     /// decompressors
-    sv_krn  : HashMap<Sv, Vec<(Kernel, Kernel, Kernel)>>,
+    sv_krn  : HashMap<sv::Sv, Vec<(Kernel, Kernel, Kernel)>>,
 }
 
 impl Decompressor {
@@ -243,21 +242,22 @@ impl Decompressor {
             sv_krn  : HashMap::new()
         }
     }
-    /// Decompresses (recovers) given CRINEX record content.   
-    /// Do not feed header data - header must be previously parsed separetely.    
+    /// Decompresses (recovers) RINEX from given CRINEX record block.   
+    /// This method will decompress and manage CRINEX comments or weird events properly.    
+    /// This method will crash on header data: header section should be previously / separately parsed.    
     /// `header` : previously identified RINEX `header` section
     /// `content`: string reference extracted from a CRINEX record.    
     ///           This method is very convenient because `content` can have any shape you can think of.    
-    ///           You can feed single CRINEX epoch lines, one at a time.     
-    ///           You can pass epochs one at a time (line groupings).   
-    ///           You can also pass an entire CRINEX record at once.    
+    ///           You can feed single CRINEX epoch lines, one at a time, just make sure it's terminated with an \n     
+    ///           You can pass epochs one at a time (line groupings, several lines at once)    
+    ///           You can also pass an entire CRINEX record at once      
     ///           You can even pass unevenly grouped chunks of epochs, even with COMMENTS unevenly inserted.    
     ///           Just make sure that `content` is never an empty string (at least 1 character),
     ///           which easily happens if you iterate through a CRINEX record with .lines() for instance.   
     ///           And also make sure to follow a standard CRINEX structure, which should always be the case   
     ///           if you iterate over a valid CRINEX.
     /// `result`: returns decompressed (recovered) block from provided block content
-    pub fn recover (&mut self, header: &header::Header, content : &str) -> Result<String, Error> {
+    pub fn decompress (&mut self, header: &header::Header, content : &str) -> Result<String, Error> {
         // Context sanity checks
         if !header.is_crinex() {
             return Err(Error::NotACrinexError)
@@ -287,6 +287,13 @@ impl Decompressor {
             };
             // [0] : COMMENTS
             if is_comment!(line) {
+                if line.contains("RINEX FILE SPLICE") {
+                    // [0*] SPLICE special comments
+                    //      merged RINEX Files
+                    //  --> reset FSM
+                    self.header = true;
+                    self.pointer = 0
+                }
                 result.push_str(line); // feed as is..
                 result.push_str("\n");
                 continue
@@ -340,27 +347,34 @@ impl Decompressor {
                         let (epoch, systems) = recovered_epoch.split_at(32);
                         result.push_str(epoch);
                         let mut begin = 0;
-                        //while begin < systems.len() {
                         let n = num_integer::div_ceil(systems.len(), 12*3); // max sv per line
-                        for i in 0..n {
+                        // terminate first line with required content
+                        let end = std::cmp::min(begin+12*3, systems.len());
+                        result.push_str(&systems[begin..end]);
+                        // squeeze clock offset here, if any
+                        if let Some(offset) = clock_offset {
+                            result.push_str(&format!("         {:3.12}", (offset as f64)/1000.0_f64))
+                        }
+                        result.push_str("\n");
+                        begin += 12*3; // `systems` pointer
+                        for i in 1..n { // nb of missing lines, to fit remaining systems 
                             let end = std::cmp::min(begin+12*3, systems.len());
-                            if i > 0 {
-                                result.push_str("                                ")
-                            }
+                            result.push_str("                                ");
                             result.push_str(&systems[begin..end]);
                             if i < n-1 {
-                                result.push_str("\n");
+                                result.push_str("\n"); // otherwise,
+                                                // last line is terminated at the end of this block
                             }
                             begin += 12*3
                         }
                     },
                     _ => { // modern RINEX
-                        result.push_str(recovered_epoch.split_at(35).0)
+                        result.push_str(recovered_epoch.split_at(35).0);
+                        if let Some(offset) = clock_offset {
+                            result.push_str(&format!("         {:3.12}", (offset as f64)/1000.0_f64))
+                        }
                     }
                 };
-                if let Some(offset) = clock_offset {
-                    result.push_str(&format!("         {:3.12}", (offset as f64)/1000.0_f64))
-                }
                 result.push_str("\n");
                 self.clock_offset = false;
                 continue
@@ -371,7 +385,6 @@ impl Decompressor {
                 .as_text()
                 .unwrap();
             let epo = recovered_epoch.as_str().trim_end();
-            //println!("EPO : \"{}\"", epo);
             let mut offset : usize =
                 2    // Y
                 +2+1 // m
@@ -389,15 +402,18 @@ impl Decompressor {
             //     ---> identify nb of satellite vehicules
             //     ---> identify which system we're dealing with
             //          using recovered header
-            //TODO <!> Attention a cet offset 41 en cas de CRINEX1
-            let offset : usize = std::cmp::min((41 + 3*(self.pointer+1)).into(), epo.len());
+            let offset : usize = match crx_version.major {
+                1 => std::cmp::min((32 + 3*(self.pointer+1)).into(), epo.len()),
+                3 => std::cmp::min((41 + 3*(self.pointer+1)).into(), epo.len()),
+                _ => return Err(Error::NonSupportedCrinexRevision)
+            };
             let system = epo.split_at(offset.into()).0;
             let system = system.split_at(system.len()-3).1; // last 3 XXX
             if rnx_version.major > 2 {
-                result.push_str(&system.to_string()); // Modern Rinex Needs this
+                result.push_str(&system.to_string()); // Modern rinex needs XXX on every line
             }
 
-            let sv = Sv::from_str(system)?;
+            let sv = sv::Sv::from_str(system)?;
             let codes = &obs_codes[&sv.constellation];
             if !self.sv_krn.contains_key(&sv) {
                 // first time dealing with this system
@@ -427,12 +443,10 @@ impl Decompressor {
             let mut rem = line.clone();
             let mut obs_data : Vec<Option<i64>> = Vec::with_capacity(12);
             loop {
-                //println!("SYSTEM ! {:?}", system);
                 if obs_count == codes.len() {
                     // FLAGS fields
                     //  ---> parse & run textdiff on each individual character
                     //   --> then format final output line
-                    //println!("FLAGS! \"{}\" | {}", rem, rem.len());
                     let mut obs_flags : Vec<String> = Vec::with_capacity(obs_data.len()*2);
                     // [+] grab all provided and apply textdiff
                     //     append BLANK in case not provided,
@@ -521,7 +535,6 @@ impl Decompressor {
                             let (order, rem) = rem.split_at(index);
                             let order = u8::from_str_radix(order.trim(),10)?;
                             let (_, data) = rem.split_at(1);
-                            //println!("ATTENTION ICI trim ??");
                             let data = i64::from_str_radix(data.trim(), 10)?;
                             let obs = self.sv_krn.get_mut(&sv)
                                 .unwrap();
@@ -594,7 +607,6 @@ impl Decompressor {
                 };
                 let (roi, r) = rem.split_at(next_wsp);
                 rem = r;
-                //println!("CODE : \"{}\" - ROI \"{}\"", codes[obs_count], roi);
                 if roi == " " { // BLANK field
                     obs_count += 1;
                     obs_data.push(None);
@@ -677,7 +689,7 @@ impl Decompressor {
 
 #[cfg(test)]
 mod test {
-    use super::*;
+    use super::{Kernel,Dtype};
     #[test]
     /// Tests numerical data recovery    
     /// through Hatanaka decompression.   
