@@ -1,9 +1,10 @@
 //! `MeteorologicalData` parser and related methods
+use std::io::Write;
 use thiserror::Error;
 use std::str::FromStr;
-use std::collections::HashMap;
-
+use std::collections::{BTreeMap, HashMap};
 use crate::epoch;
+use crate::header;
 use crate::header::Header;
 
 #[macro_export]
@@ -79,7 +80,7 @@ macro_rules! is_hail_indicator_code {
 }
 
 /// `Record` content for Meteo data files
-pub type Record = HashMap<epoch::Epoch, HashMap<String, f32>>;
+pub type Record = BTreeMap<epoch::Epoch, HashMap<String, f32>>;
 
 #[derive(Error, Debug)]
 /// Meteo Data `Record` parsing specific errors
@@ -93,8 +94,8 @@ pub enum RecordError {
 }
 
 /// Builds `RINEX` record entry for `Meteo` Data files
-pub fn build_record_entry (header: &Header, content: &str)
-    -> Result<(epoch::Epoch, HashMap<String, f32>), RecordError> 
+pub fn build_record_entry (header: &Header, content: &str) 
+        -> Result<(epoch::Epoch, HashMap<String, f32>), RecordError> 
 {
     let mut lines = content.lines();
     let mut line = lines.next()
@@ -102,18 +103,6 @@ pub fn build_record_entry (header: &Header, content: &str)
 
 	let mut map : HashMap<String, f32> = HashMap::with_capacity(3);
 
-	//let mut offset : usize = 
-    //    2+1 // Y
-    //    +2+1 // d
-    //    +2+1 // m
-    //    +2+1 // h
-    //    +2+1 // m
-    //    +2+1; // secs is not f32 as usual
-    
-	//if header.version.major > 2 {
-	//	offset += 2;
-	//}
-
 	// epoch.secs is not f32 as usual
 	// Y is 4 digit number as usual for V > 2
 	//let (date, rem) = line.split_at(offset);
@@ -187,6 +176,18 @@ pub fn build_record_entry (header: &Header, content: &str)
 			}
 		}
 	} // nb lines
-
 	Ok((epoch, map))
 }
+
+/// Pushes meteo record into given file writer
+pub fn to_file (header: &header::Header, record: &Record, mut writer: std::fs::File) -> std::io::Result<()> {
+    let obs_codes = header.met_codes.as_ref().unwrap();
+    for epoch in record.keys() {
+        write!(writer, " {} ", epoch.date.format("%y %_m %_d %_H %_M %_S").to_string())?;
+        for code in obs_codes { 
+            write!(writer, "{:.1}   ", record[epoch].get(code).unwrap())?;
+        }
+        write!(writer, "\n")?
+    }
+    Ok(())
+}
